#include <Wire.h>
#include <string.h>

#undef int
#include <stdio.h>

uint8_t outbuf[6];		// array to store arduino output
int cnt = 0;
int ledPin = 13;
const int pwmPins[] = {3, 5, 6, 9};
int pwmVals[] = {0, 0, 0, 0};
int xval = 0;
int yval = 0;
int xmin = 250;
int ymin = 250;
int xmax = 650;
int ymax = 650;
boolean resetCalib = false;

int joy_x_axis = 128;
int joy_y_axis = 128;
int accel_x_axis = 512; 
int accel_y_axis = 512;
int accel_z_axis = 512;
int z_button = 0;
int c_button = 0;

void setup ()
{
  for (int i=0; i<4; i++) {
    pinMode(pwmPins[i], OUTPUT);
  }
  
  Serial.begin(19200);
  Wire.begin ();		// join i2c bus with address 0x52
  nunchuck_init ();             // send the initilization handshake
  Serial.print ("Finished setup\n");
}

void nunchuck_init ()
{
  Wire.beginTransmission (0x52);  // transmit to device 0x52
  Wire.send (0x40);		  // sends memory address
  Wire.send (0x00);		  // sends sent a zero.  
  Wire.endTransmission ();	  // stop transmitting
}

void send_zero ()
{
  Wire.beginTransmission (0x52);  // transmit to device 0x52
  Wire.send (0x00);		  // sends one byte
  Wire.endTransmission ();	  // stop transmitting
}

void loop ()
{
  Wire.requestFrom (0x52, 6);	// request data from nunchuck
  while (Wire.available ())
    {
      outbuf[cnt] = nunchuk_decode_byte (Wire.receive ());	// receive byte as an integer
      digitalWrite (ledPin, HIGH);	// sets the LED on
      cnt++;
    }

  // If we recieved the 6 bytes, then go print them
  if (cnt >= 5)
    {
      update();
      for (int i=0; i<4; i++) {
        analogWrite(pwmPins[i], pwmVals[i]);
      }
    }

  cnt = 0;
  send_zero (); // send the request for next bytes
  delay (10);
}

// Print the input data we have recieved accel data is 10 bits long
// so we read 8 bits, then we have to add on the last 2 bits.  That is why I
// multiply them by 2 * 2
void update()
{
  joy_x_axis = outbuf[0];
  joy_y_axis = outbuf[1];
  accel_x_axis = outbuf[2] * 2 * 2; 
  accel_y_axis = outbuf[3] * 2 * 2;
  accel_z_axis = outbuf[4] * 2 * 2;
  z_button = 0;
  c_button = 0;

 // byte outbuf[5] contains bits for z and c buttons it also contains the least significant bits for the accelerometer data
 // so we have to check each bit of byte outbuf[5]
  if ((outbuf[5] >> 2) & 1) {
      accel_x_axis += 2;
    }
  if ((outbuf[5] >> 3) & 1) {
      accel_x_axis += 1;
    }
  if ((outbuf[5] >> 4) & 1) {
      accel_y_axis += 2;
    }
  if ((outbuf[5] >> 5) & 1) {
      accel_y_axis += 1;
    }
  if ((outbuf[5] >> 6) & 1) {
      accel_z_axis += 2;
    }
  if ((outbuf[5] >> 7) & 1) {
      accel_z_axis += 1;
    }
    
  xmin += (joy_x_axis-121)/25;
  xmax += (joy_x_axis-121)/25;
  ymin += (joy_y_axis-131)/25;
  ymax += (joy_y_axis-131)/25;
    
  // Calculate output for the four sensors normally
  xval = max((unsigned long)(min((unsigned int)accel_x_axis,xmax)-xmin)*255/(xmax-xmin),0);
  yval = max((unsigned long)(min((unsigned int)accel_y_axis,ymax)-ymin)*255/(ymax-ymin),0);
  if (xval<0) { xval=0; }
  if (yval<0) { yval=0; }
  if (xval>255) { xval=255; }
  if (yval>255) { yval=255; }
  yval=255-yval;
  pwmVals[0] = ((255-yval) + (xval))/2;
  pwmVals[1] = ((yval) + (xval))/2;
  pwmVals[2] = ((255-yval) + (255-xval))/2;
  pwmVals[3] = ((yval) + (255-xval))/2;

  if ((outbuf[5] >> 0) & 1) {
      z_button = 1;
  } else {
    // jump
    pwmVals[0] = 0;
    pwmVals[1] = 0;
    pwmVals[2] = 0;
    pwmVals[3] = 0;
  }
  if ((outbuf[5] >> 1) & 1) {
      c_button = 1;
      resetCalib = true;
  } else {
      if (resetCalib) {
        xmax = accel_x_axis;
        xmin = accel_x_axis;
        ymax = accel_y_axis;
        ymin = accel_y_axis;
        resetCalib = false;
      } else {
        // calibrate
        if (accel_x_axis>xmax) {
          xmax = accel_x_axis;
        }
        if (accel_x_axis<xmin) {
          xmin = accel_x_axis;
        }
        if (accel_y_axis>ymax) {
          ymax = accel_y_axis;
        }
        if (accel_y_axis<ymin) {
          ymin = accel_y_axis;
        }
      }
  }

  Serial.print (joy_x_axis, DEC);
  Serial.print ("\t");

  Serial.print (joy_y_axis, DEC);
  Serial.print ("\t");

  Serial.print (accel_x_axis);
  Serial.print ("\t");

  Serial.print (accel_y_axis);
  Serial.print ("\t");

  Serial.print (accel_z_axis, DEC);
  Serial.print ("\t");

  Serial.print (z_button, DEC);
  Serial.print ("\t");

  Serial.print (c_button, DEC);
  Serial.print ("\t");
 
  Serial.print (pwmVals[0], DEC);
  Serial.print ("\t");
  Serial.print (pwmVals[1], DEC);
  Serial.print ("\t");
  Serial.print (pwmVals[2], DEC);
  Serial.print ("\t");
  Serial.print (pwmVals[3], DEC);
  Serial.print ("\t");

  Serial.print (xmin, DEC);
  Serial.print ("\t");
  Serial.print (xmax, DEC);
  Serial.print ("\t");
  Serial.print (ymin, DEC);
  Serial.print ("\t");
  Serial.print (ymax, DEC);
  Serial.print ("\t");

  Serial.print (xval, DEC);
  Serial.print ("\t");
  Serial.print (yval, DEC);
  Serial.print ("\t");

  Serial.print ("\r\n");
}

// Encode data to format that most wiimote drivers except only needed if you use one of the regular wiimote drivers
char nunchuk_decode_byte (char x)
{
  x = (x ^ 0x17) + 0x17;
  return x;
}
